#!/bin/sh

# copy the top and lower level update.sh scripts over so we can rebuild a bundle in the future. 
copyForPfu()
{
    mkdir -p /cfg/pfu/cfg/components

    cp /mnt/ramdisk/cfg/*.sh /cfg/pfu/cfg
}

msg()
{
    echo $1
}

stopApps()
{

    echo "Disabling soft watchdogs..."
    disablewatchdogs
    sleep 3
    echo "Killing watcherd"
    killall -9 watcherd
    echo "Disabling moose watchdog"
    echo "0" > /proc/moose/watchdog/timeout-secs
    shields up;    # kills watchdog timer
    if [ -f /usr/bin/killappsnorpc ]
	then
       killappsnorpc
	else
	   killapps
	fi

    # start secondary webserver
    # begin streaming progress messages out of it

    sleep 2 # allow system to clean up
    return
}

startMsgServer()
{
    msg "Starting message server"
    httpdlite.srv start

    return
}

readsi()
{
if [ $1 == "A" ]
then
    echo `xmlcntrl /cfg/sysinfo/system.xml MCAInfo $2`
else
    echo `xmlcntrl /cfg/sysinfo/system.xml MCBInfo $2`
fi
}

writesi()
{
if [ $1 == "A" ]
then
    echo `xmlcntrl /cfg/sysinfo/system.xml MCAInfo $2 $3`
else
    echo `xmlcntrl /cfg/sysinfo/system.xml MCBInfo $2 $3`
fi
}

if [ ! -f "/mnt/ramdisk/is.cfg" ]
then
    copyForPfu
    exit 0
fi

stopApps

startMsgServer

if [ -d "/cfg/etc/autopool" ]
then
    msg "Clearing current configuration... custom autopool rules"
    rm -rf /cfg/etc/autopool
fi

if [ -d "/cfg/brand" ]
then
    msg "Clearing current configuration... branding"
    rm -rf /cfg/brand
fi

#I feel like users and roles are things we need to keep if the other side didn't have them to give us...
if [ -d "/mnt/ramdisk/cfg/components/users" ] && [ -d "/mnt/ramdisk/cfg/components/roles" ]
then
    msg "Clearing current configuration... users"
    rm -f /cfg/users/*
    rm -f /cfg/roles/*
fi

msg "Clearing current configuration... schedules and tasks"
rm -f /cfg/scheds/*
rm -f /cfg/tasks/*

msg "Clearing current configuration... remote systems"
rm -f /cfg/rs/*

msg "Clearing current configuration... snmp"
rm /cfg/etc/snmpd.conf
rm /cfg/etc/config/snmpd.conf

msg "Clearing current configuration... smis"
rm -rf /cfg/opt/*
rm -rf /cfg/sfcb/*

if [ -d "/cfg/etc/health" ]
then
    msg "Clearing current configuration... health"
    rm -rf /cfg/etc/health
fi

msg "Clearing current configuration... ssl certificate"
rm -f /cfg/ssl/*

rm -f /cfg/ssh/sshd_config

if [ -d "/mnt/ramdisk/cfg/components/etc/autopool" ]
then
    if [ ! -d "/cfg/etc/autopool" ]
    then
        mkdir /cfg/etc/autopool
    fi
    msg "Applying configuration... custom autopool rules"
    cp -R /mnt/ramdisk/cfg/components/etc/autopool/* /cfg/etc/autopool
fi

if [ -d "/mnt/ramdisk/cfg/components/brand" ]
then
    if [ ! -d "/cfg/brand" ]
    then
        mkdir /cfg/brand
    fi
    msg "Applying configuration... branding"
    cp -R /mnt/ramdisk/cfg/components/brand/* /cfg/brand
fi

msg "Applying configuration... users"
cp /mnt/ramdisk/cfg/components/users/* /cfg/users
cp /mnt/ramdisk/cfg/components/roles/* /cfg/roles

msg "Applying configuration... schedules and tasks"
cp /mnt/ramdisk/cfg/components/scheds/* /cfg/scheds
cp /mnt/ramdisk/cfg/components/tasks/* /cfg/tasks

msg "Applying configuration... remote systems"
cp /mnt/ramdisk/cfg/components/rs/* /cfg/rs

msg "Applying configuration... snmp"
cp /mnt/ramdisk/cfg/components/etc/snmpd.conf /cfg/etc
cp /mnt/ramdisk/cfg/components/etc/config/snmpd.conf /cfg/etc/config

msg "Applying configuration... smis"
cp -R /mnt/ramdisk/cfg/components/opt/* /cfg/opt
cp -R /mnt/ramdisk/cfg/components/sfcb/* /cfg/sfcb

if [ -d "/mnt/ramdisk/cfg/components/etc/health" ]
then
    msg "Applying configuration... health"
    if [ ! -d "/cfg/etc/health" ]
    then
        mkdir /cfg/etc/health
    fi
    cp -R /mnt/ramdisk/cfg/components/etc/health/* /cfg/etc/health
fi

msg "Applying configuration... ssl certificate"
cp /mnt/ramdisk/cfg/components/ssl/* /cfg/ssl

cp /mnt/ramdisk/cfg/components/ssh/sshd_config /cfg/ssh

msg "Modifying configuration... system information"
mc=`readslot`
MACADDRESS=$(readsi $mc macAddress)
BUNDLEVERSION=$(readsi $mc bundleVersion)
BUNDLEDATE=$(readsi $mc bundleBuildDate)
MCVERSION=$(readsi $mc mcVersion)
CPLDVERSION=$(readsi $mc cpldVersion)
UNIQUENAME=$(readsi $mc uniqueName)
CONTROLLERSERIAL=$(readsi $mc controllerSerial)
LASTCLTIME=$(readsi $mc lastCodeloadTime)
LASTPAIRMIDPLANE=$(readsi $mc lastPairedMidplaneSerial)
# Rick and Robert: removing this because we push the value with sysinfo
# LASTPAIRPARTNER=$(readsi $mc lastPairedPartnerSerial)

cp /mnt/ramdisk/cfg/components/sysinfo/system.xml /cfg/sysinfo

writesi $mc macAddress $MACADDRESS
writesi $mc bundleVersion $BUNDLEVERSION
writesi $mc bundleBuildDate $BUNDLEDATE
writesi $mc mcVersion $MCVERSION
writesi $mc cpldVersion $CPLDVERSION
writesi $mc uniqueName $UNIQUENAME
writesi $mc controllerSerial $CONTROLLERSERIAL
writesi $mc lastCodeloadTime $LASTCLTIME
writesi $mc lastPairedMidplaneSerial $LASTPAIRMIDPLANE
# Rick and Robert: removing this because we push the value with sysinfo
# writesi $mc lastPairedPartnerSerial $LASTPAIRPARTNER

msg "Finished updating Management Controller configuration"

copyForPfu


sync

exit 0

