
# Required Parameters: 
# 	Version strings (eg: TS100A17 TS200R55) 
#
# Returns: 
#  1 if A is greater than B
#  -1 if A is less than B
#  0 if they are the same
# 
# Example: 	echo "TS100A17 TS122A15" | awk -f <thisfile.awk>
#
# TS		Bundle Family, or "FAM" (TS, L, etc)
# 100: 		Bundle Level, or "LVL"
# B: 		Maturity Designation, or "MAT"
# 001: 		Revision, or "REV"

function parse( a,s )
{
    p=match(a, /^[a-zA-Z]+/)
    family=substr(a,p,RLENGTH)
	split( substr(a,RLENGTH+1,length(a)),b,/[a-zA-Z]/)
	level=b[1]
	revision=b[2]
	p=substr(a,RLENGTH+1,length(a))
	pos=match(p,/[a-zA-Z]+/)
	maturity=substr(p,pos,RLENGTH)

	if (s=="FAM")
		return family

	if (s=="LVL")
		return level

	if (s=="MAT")
		return maturity

	if (s=="REV")
		return revision
}

function compare_level( a,b )
{
	#families must be the same or the level comparison is irrelevant
	#otherwise, for example, P100 codeloads are prevented in T240 systems.
        if (parse(a,"FAM") != parse(b,"FAM"))              
                return 0;                                 
                                                          
   	if (parse(a,"LVL") > parse(b,"LVL"))
		return 1
	else if (parse(a,"LVL") < parse(b,"LVL"))
		return -1
	else
		return 0
}

{ print compare_level($1,$2) } 


