#!/bin/sh

actComp="/bundle/ec"
if [ -f /usr/bin/utils.sh ]; then
    source /usr/bin/utils.sh
else
    apiOK() { :; }
    apiNA() { :; }
    apiActive() { :; }
    apiActiveEcho() { :; }
    apiProgress() { :; }
    apiError() { :; }
fi

source /app/etc/app.env

apiActive $actComp "Loading EC firmware."
echo "STATUS: Updating Expander Controller firmware."

dirName="components"
sysInfoFilename="/cfg/sysinfo/system.xml"
pfu="--pfu" # default: use the PFU option

if [ -d "$dirName" ]; then
    fullDirName=$(readlink -f $dirName)
    pfuDisabled=`xmlcntrl $sysInfoFilename MEMBER pfuDisabled`

    if [ $pfuDisabled == "true" ]; then # PFU is DISABLED
        pfu="" # don't load alternate path
    fi

    # The potential retries are necessary to deal with a very
    # intermittent EC hang, which causes an EC load failure.
    retryCount=1
    rc=1
    while [ $rc -ne 255 ] && [ $rc -ne 0 ] && [ $retryCount -le 3 ]
    do
        echo "EMP Load Attempt $retryCount"
        ../capi.app --loadec $fullDirName/ --component $actComp $pfu
        rc=$?
        retryCount=$((retryCount+1))
    done

    clireturn="$rc"

    echo "Finished updating EC firmware. [$clireturn]"
    rm -f $fullFlaName
    if [ "$clireturn" -eq "0" ]; then
        apiOK $actComp
        exit $LOAD_STATUS_SUCCESS
    elif [ "$clireturn" -eq "255" ]; then
        apiNA $actComp
        exit $LOAD_STATUS_NOT_ATTEMPTED
    else
        apiError $actComp 1300 "EC code load failed."
        exit $LOAD_STATUS_FAILED
    fi
else
    apiNA $actComp
    echo "No EC firmware to update."
    exit $LOAD_STATUS_NOT_ATTEMPTED
fi

