#!/bin/sh

source ${PWD}/utils.sh
source ${PWD}/fcns.shinc;   # use local copy instead of /usr/bin/fncs.shinc

export FLASHVER_TGT=$(inicntrl ${PWD}/version.txt MC_system flash_layout)
export FLASHVER_CUR=$(inicntrl /cfg/version.txt.prv MC_system flash_layout)

export BUNDLEINTEGRITY=1
export MIGRATEFLASH=0

actComp="bundle/mc"

TARGETED_MC_VERSION=$(inicntrl ${PWD}/version.txt MC_system  version)
CURRENT_MC_VERSION=$(inicntrl /cfg/version.txt.prv MC_system  version)

apiActive $actComp "Loading MC firmware."
msg "STATUS: Updating Management Controller firmware"
msg "MC current version: $CURRENT_MC_VERSION,loading version $TARGETED_MC_VERSION"

shields up

# blockDowngrade

checkForMigration

stopApps

unmountApps

startMsgServer

# L20* did not mknod all mtd parts
# mdev -s

keygenCheck

# =====================================================================
COMPONENT="$actComp/loader"
apiActive $actComp "Loading MC Boot Loader."

# Following creates variables like:  mtdnbr_app, mtdnbr_log, mtdnbr_cfg, etc.
genAllMtdPropVariables "$(cat /proc/moose/mtd)"

if [ -f components/boot_data.bin ]; then
    loadmtd "${PWD}/components/boot_data.bin" \
            "$mtdnbr_loader"       $COMPONENT \
            loadSuccess
    $loadSuccess || exit $LOAD_STATUS_FAILED
else
    apiNA $COMPONENT
fi

# =====================================================================
COMPONENT="$actComp/meta"
apiActive $actComp "Loading MC Boot Loader Meta."
if [ -f components/boot_meta.bin ]; then
    loadmtd "${PWD}/components/boot_meta.bin" \
            "$mtdnbr_loader_meta"  $COMPONENT \
            loadSuccess
    $loadSuccess || exit $LOAD_STATUS_FAILED
else
    apiNA $COMPONENT
fi

# Gallium now interprets boot_data.bin and copies the correct
# chunks into MC SPI flash, too.  This was done as a reliability
# improvement vs. the bootloader doing the copying itself.
flashSpi=true
[ ! -f components/boot_data.bin ] && flashSpi=false
[ ! -f components/boot_meta.bin ] && flashSpi=false
[ -z "$mtdnbr_bootloader" ]       && flashSpi=false
[ -z $(which bl-metaprops) ]      && flashSpi=false
[ -z $(which cp-to-mtd) ]         && flashSpi=false
[ ! -f /proc/moose/mcfu/cmd ]     && flashSpi=false

if $flashSpi; then
    # "source" all the boot_meta.bin properties as variables
    eval "$(bl-metaprops components/boot_meta.bin)"
fi

itsOlder="Using older flash layout.  This component is not applicable"

# =====================================================================
COMPONENT="$actComp/mcspiprep"
apiActiveEcho $actComp "Prepare MC SPI Flash for Update."
if $flashSpi; then
    if [ $bl_MC_num_volumes -ge 2 ]; then

        maybeEraseSpiFlash "MC" $mtdnbr_bootloader "$COMPONENT" \
            $((0x$mtdsize_bootloader/(1024*1024))) flashWasErased loadSuccess

        # If erased, tell main script that we need to do a full system reboot
        talkToCaller $flashWasErased
    else
        apiError "$COMPONENT" 1209 "Failure to Prepare $COMPONENT due to change in boot_meta.bin format"
        exit $LOAD_STATUS_FAILED
    fi
else
    apiNA $COMPONENT "$itsOlder"
fi

# =====================================================================
COMPONENT="$actComp/ramloader"
apiActive $actComp "Loading MC RAM Loader SPI Flash."
if $flashSpi; then
    if [ $bl_MC_num_volumes -ge 1 ]; then

        useCptomtd "${PWD}/components/boot_data.bin" \
            "$mtdnbr_bootloader" $COMPONENT \
            "$bl_MC_vol0_src_base" "$bl_MC_vol0_dest_base" "$bl_MC_vol0_src_length" \
            loadSuccess

        $loadSuccess || exit $LOAD_STATUS_FAILED
    else
        apiError "$COMPONENT" 1209 "Failure to Flash $COMPONENT due to change in boot_meta.bin format"
        exit $LOAD_STATUS_FAILED
    fi
else
    apiNA $COMPONENT "$itsOlder"
fi

# =====================================================================
COMPONENT="$actComp/bootloader"
apiActive $actComp "Loading MC Boot Loader SPI Flash."

if $flashSpi; then
    if [ $bl_MC_num_volumes -ge 2 ]; then

        useCptomtd "${PWD}/components/boot_data.bin" \
            "$mtdnbr_bootloader" $COMPONENT \
            "$bl_MC_vol1_src_base" "$bl_MC_vol1_dest_base" "$bl_MC_vol1_src_length" \
            loadSuccess

        $loadSuccess || exit $LOAD_STATUS_FAILED
    else
        apiError "$COMPONENT" 1209 "Failure to Flash $COMPONENT due to change in boot_meta.bin format"
        exit $LOAD_STATUS_FAILED
    fi
else
    apiNA $COMPONENT "$itsOlder"
fi

# =====================================================================
fuFlashSpi=false
if $flashSpi; then
    isMcfuWorking fuFlashSpi mbToErase

    if $fuFlashSpi; then
        # MC-FU workaround
        maybeReplaceMcfuDriver  fuFlashSpi
    fi
fi
# =====================================================================


# =====================================================================
COMPONENT="$actComp/fuspiprep"
apiActiveEcho $actComp "Prepare FU SPI Flash for Update."
if $fuFlashSpi; then
    if [ $bl_FU_num_volumes -ge 2 ]; then

        maybeEraseSpiFlash "FU" $MCFU_FAKE_MTDNBR "$COMPONENT" \
            $mbToErase flashWasErased loadSuccess

        # If erased, tell main script that we need to do a full system reboot
        talkToCaller $flashWasErased
    else
        apiError "$COMPONENT" 1209 "Failure to Prepare $COMPONENT due to change in boot_meta.bin format"
        exit $LOAD_STATUS_FAILED
    fi
else
    apiNA $COMPONENT "$itsOlder"
fi

# =====================================================================
COMPONENT="$actComp/furamloader"
apiActive $actComp "Loading FU RAM Loader SPI Flash."
if $fuFlashSpi; then
    if [ $bl_FU_num_volumes -ge 1 ]; then

        useCptomtd "${PWD}/components/boot_data.bin" \
            "$MCFU_FAKE_MTDNBR" $COMPONENT \
            "$bl_FU_vol0_src_base" "$bl_FU_vol0_dest_base" "$bl_FU_vol0_src_length" \
            loadSuccess

        $loadSuccess || exit $LOAD_STATUS_FAILED
    else
        apiError "$COMPONENT" 1209 "Failure to Flash $COMPONENT due to change in boot_meta.bin format"
        exit $LOAD_STATUS_FAILED
    fi
else
    apiNA $COMPONENT "$itsOlder"
fi

# =====================================================================
COMPONENT="$actComp/fubootloader"
apiActive $actComp "Loading FU Boot Loader SPI Flash."

if $fuFlashSpi; then
    if [ $bl_FU_num_volumes -ge 2 ]; then

        useCptomtd "${PWD}/components/boot_data.bin" \
            "$MCFU_FAKE_MTDNBR" $COMPONENT \
            "$bl_FU_vol1_src_base" "$bl_FU_vol1_dest_base" "$bl_FU_vol1_src_length" \
            loadSuccess

        $loadSuccess || exit $LOAD_STATUS_FAILED
    else
        apiError "$COMPONENT" 1209 "Failure to Flash $COMPONENT due to change in boot_meta.bin format"
        exit $LOAD_STATUS_FAILED
    fi
else
    apiNA $COMPONENT "$itsOlder"
fi


# boot loader needs to know exact size of zImage, initramfs
bootcontents=$(cat /dev/mtd$mtdnbr_mc_boot_meta | strings)
getCurImageSizes "$bootcontents" sizeKernel sizeInitramfs

# =====================================================================
COMPONENT="$actComp/kernel"
apiActive $actComp "Loading MC Kernel."
makeNewBootcmd=false
if [ -f components/zImage ]; then
    loadmtd "${PWD}/components/zImage" \
            "$mtdnbr_kernel"       $COMPONENT \
            loadSuccess
    $loadSuccess || exit $LOAD_STATUS_FAILED
    hexSize "${PWD}/components/zImage"          sizeKernel
    makeNewBootcmd=true
else
    apiNA $COMPONENT
fi

# =====================================================================
COMPONENT="$actComp/rootfs"
apiActive $actComp "Loading MC Root Filesystem."
if [ -f components/initramfs.gz ]; then
    loadmtd "${PWD}/components/initramfs.gz" \
            "$mtdnbr_initramfs"    $COMPONENT \
            loadSuccess
    $loadSuccess || exit $LOAD_STATUS_FAILED
    hexSize "${PWD}/components/initramfs.gz"    sizeInitramfs
    makeNewBootcmd=true
else
    apiNA $actComp
fi

if $makeNewBootcmd; then
    createBootString    "0x$mtdoffset_kernel"       "0x$sizeKernel" \
                        "0x$mtdoffset_initramfs"    "0x$sizeInitramfs" \
                        "Upgrade bundle" \
                        mybootstring

    echo -e "$mybootstring" > /mnt/ramdisk/mc_boot_meta.txt

    # Used by memtracker if it notices the
    # bootcmd missing from /dev/mtd0
    cp /mnt/ramdisk/mc_boot_meta.txt /cfg

    COMPONENT="$actComp/bootmeta"
    apiActive $actComp "Loading MC Boot Meta Data."

    loadmtd "/mnt/ramdisk/mc_boot_meta.txt" \
            "$mtdnbr_mc_boot_meta" $COMPONENT \
            loadSuccess
    $loadSuccess || exit $LOAD_STATUS_FAILED
fi

# =====================================================================
COMPONENT="$actComp/apps"
apiActive $actComp "Loading MC Applications."
if [ -f components/app.squashfs ]; then
    loadmtd "${PWD}/components/app.squashfs" \
            "$mtdnbr_app"          $COMPONENT \
            loadSuccess
    $loadSuccess || exit $LOAD_STATUS_FAILED
else
    apiNA $actComp
fi

if [ "$MIGRATEFLASH" -gt "0" ]; then
    if [ -z "/dev/mtd$mtdnbr_cfg" ]; then
        msg "ERROR: Cannot upgrade cfg.  No match for mtd partition name."
    else
        migrateCfg "$mtdnbr_cfg"
    fi
fi

apiActiveEcho $actComp "Finished updating Management Controller firmware"

if [ "$BUNDLEINTEGRITY" -eq "1" ]
then
    msg "Updating system configuration files"
    # if not there, remount the CFG mtd mount to be able to copy files to it
    if [ $(cfgMounted) == "0" ]
    then
        msg "Re-mounting CFGFS"
        mount -t jffs2 "/dev/mtd$mtdnbr_cfg" /mnt/cfg
    fi

    # copy the version.txt over to the JFFS2
    # in the normal course of events, we've already done this
    if [ "$USE_SYSTEM_VERSION" != "true" ]
    then
	    if [ -f /cfg/version.txt ]
	    then
            mv /cfg/version.txt /cfg/version.txt.prv
	    fi
        cp ${PWD}/version.txt /cfg/version.txt
    fi

    copyForPfu

    # /proc/moose/cpld/midplane_sn stores the serial number with a leading 0x, this is fine because
    # when we retrieve it in the synchronator, it will look exactly the same
    midplane=$(cat /proc/moose/cpld/midplane_sn)
    echo $midplane > /cfg/burn2active/MidPlaneSerNum
    ts=`date +%s`
    mc=`readslot`

    if [ "$mc" == "A" ]
    then
        xmlcntrl /cfg/sysinfo/system.xml MCAInfo loadMidplaneSerial $midplane
        xmlcntrl /cfg/sysinfo/system.xml MCAInfo lastLoadedTime $ts
    else
        xmlcntrl /cfg/sysinfo/system.xml MCBInfo loadMidplaneSerial $midplane
        xmlcntrl /cfg/sysinfo/system.xml MCBInfo lastLoadedTime $ts
    fi

    if [ "$USE_SYSTEM_VERSION" == "true" ]
    then
        cp /mnt/ramdisk/system/version.txt /cfg/pfu/mc
    else
        cp ${PWD}/version.txt /cfg/pfu/mc
    fi
    msg "System configuration complete"
else
    echo "WARNING: This update process determined the software bundle was corrupt."
    echo "Although a partial load took place, you should immediately retry your software update request."

    apiError $actComp 1200 "Bundle Corrupt."
    exit $LOAD_STATUS_FAILED
fi

sync

apiOK $actComp
exit $LOAD_STATUS_SUCCESS

